with Ada.Calendar.Formatting;
with Ada.Calendar.Time_Zones;
with Ada.Hierarchical_File_Names;
procedure Birdeyes.Pages.View_Page (
	Output : not null access Ada.Streams.Root_Stream_Type'Class;
	Template : in String;
	Message : in String := "";
	View_User_Id : in String;
	Watching : in Users.Watching_Syndications.List;
	Watched_Syndications : in
		Syndications.Syndication_Info_And_Articles_Lists.List;
	User_Id : in String)
is
	use type Users.Showing_Mode;
	procedure Handle (
		Output : not null access Ada.Streams.Root_Stream_Type'Class;
		Tag : in String;
		Contents : in Web.Producers.Template) is
	begin
		if Tag = "title" then
			Web.Write_In_HTML (Output, Web.XHTML, View_User_Id & "'s antenna");
		elsif Tag = "message" then
			if Message'Length /= 0 then
				declare
					procedure Handle (
						Output : not null access Ada.Streams.Root_Stream_Type'Class;
						Tag : in String;
						Contents : in Web.Producers.Template) is
					begin
						if Tag = "text" then
							Web.Write_In_HTML (Output, Web.XHTML, Message);
						else
							Raise_Unknown_Tag (Tag);
						end if;
					end Handle;
				begin
					Web.Producers.Produce (Output, Contents, Handler => Handle'Access);
				end;
			end if;
		elsif Tag = "userpanel" then
			if User_Id'Length /= 0 then
				declare
					procedure Handle (
						Output : not null access Ada.Streams.Root_Stream_Type'Class;
						Tag : in String;
						Contents : in Web.Producers.Template) is
					begin
						if Tag = "id" then
							Web.Write_In_HTML (Output, Web.XHTML, User_Id);
						else
							Raise_Unknown_Tag (Tag);
						end if;
					end Handle;
				begin
					Web.Producers.Produce (Output, Contents, "logon", Handler => Handle'Access);
				end;
			else
				Web.Producers.Produce (Output, Contents, "logoff");
			end if;
		elsif Tag = "syndication" then
			declare
				Time_Offset : constant Ada.Calendar.Time_Zones.Time_Offset :=
					Ada.Calendar.Time_Zones.UTC_Time_Offset;
				function Image (Time : Ada.Calendar.Time) return String is
				begin
					return Ada.Calendar.Formatting.Image (Time, Time_Zone => Time_Offset)
						& Ada.Calendar.Formatting.Image (Time_Offset);
				end Image;
			begin
				for I in Watched_Syndications.Iterate loop
					declare
						Watched_Syndication : Syndications.Syndication_Info_And_Articles
							renames Watched_Syndications.Constant_Reference (I);
						procedure Handle (
							Output : not null access Ada.Streams.Root_Stream_Type'Class;
							Tag : in String;
							Contents : in Web.Producers.Template) is
						begin
							if Tag = "title" then
								Web.Write_In_HTML (
									Output,
									Web.XHTML,
									Watched_Syndication.Info.Title.Constant_Reference);
							elsif Tag = "author" then
								if Watched_Syndication.Info.Author.Is_Null then
									Web.Write_In_HTML (Output, Web.XHTML, "unknown");
								else
									Web.Write_In_HTML (
										Output,
										Web.XHTML,
										Watched_Syndication.Info.Author.Constant_Reference);
								end if;
							elsif Tag = "version" then
								Web.Write_In_HTML (
									Output,
									Web.XHTML,
									Syndications.Image (Watched_Syndication.Info.Version));
							elsif Tag = "uri" then
								Web.Write_In_HTML (
									Output,
									Web.XHTML,
									Watched_Syndication.Info.URI.Constant_Reference);
							elsif Tag = "lmt" then
								Web.Write_In_HTML (
									Output,
									Web.XHTML,
									Image (Syndications.Last_Modified_Time (Watched_Syndication.Info)));
							elsif Tag = "lat" then
								Web.Write_In_HTML (
									Output,
									Web.XHTML,
									Image (Watched_Syndication.Info.Last_Avail_Time));
							elsif Tag = "lct" then
								Web.Write_In_HTML (
									Output,
									Web.XHTML,
									Image (Watched_Syndication.Info.Last_Tried_Time));
							elsif Tag = "latest" then
								if Watching.Constant_Reference (
								         Users.Find (
								            Watching,
								            Watched_Syndication.Info.URI.Constant_Reference))
								      .Showing =
								   Users.Latest
									and then not Watched_Syndication.Articles.Is_Empty
								then
									declare
										Article : Syndications.Article
											renames Watched_Syndication.Articles.Constant_Reference (
												Watched_Syndication.Articles.First);
										procedure Handle (
											Output : not null access Ada.Streams.Root_Stream_Type'Class;
											Tag : in String;
											Contents : in Web.Producers.Template) is
										begin
											if Tag = "lpt" then
												Web.Write_In_HTML (
													Output,
													Web.XHTML,
													Image (Article.Time));
											else
												Raise_Unknown_Tag (Tag);
											end if;
										end Handle;
									begin
										Web.Producers.Produce (Output, Contents, Handler => Handle'Access);
									end;
								end if;
							elsif Tag = "expanded" then
								if Watching.Constant_Reference (
								         Users.Find (
								            Watching,
								            Watched_Syndication.Info.URI.Constant_Reference))
								      .Showing =
								   Users.Expanded
								then
									declare
										procedure Handle (
											Output : not null access Ada.Streams.Root_Stream_Type'Class;
											Tag : in String;
											Contents : in Web.Producers.Template) is
										begin
											if Tag = "article" then
												for J in Watched_Syndication.Articles.Iterate loop
													declare
														Article : Syndications.Article
															renames Watched_Syndication.Articles.Constant_Reference (J);
														procedure Handle (
															Output : not null access Ada.Streams.Root_Stream_Type'Class;
															Tag : in String;
															Contents : in Web.Producers.Template) is
														begin
															if Tag = "title" then
																Web.Write_In_HTML (
																	Output,
																	Web.XHTML,
																	Article.Title.Constant_Reference);
															elsif Tag = "pt" then
																Web.Write_In_HTML (
																	Output,
																	Web.XHTML,
																	Image (Article.Time));
															elsif Tag = "href_link" then
																Web.Write_Begin_Attribute (Output, "href");
																Web.Write_In_Attribute (
																	Output,
																	Web.XHTML,
																	Article.Link.Constant_Reference);
																Web.Write_End_Attribute (Output);
															else
																Raise_Unknown_Tag (Tag);
															end if;
														end Handle;
													begin
														Web.Producers.Produce (Output, Contents, Handler => Handle'Access);
													end;
												end loop;
											else
												Raise_Unknown_Tag (Tag);
											end if;
										end Handle;
									begin
										Web.Producers.Produce (Output, Contents, Handler => Handle'Access);
									end;
								end if;
							elsif Tag = "href_uri" then
								Web.Write_Begin_Attribute (Output, "href");
								Web.Write_In_Attribute (
									Output,
									Web.XHTML,
									Watched_Syndication.Info.URI.Constant_Reference);
								Web.Write_End_Attribute (Output);
							elsif Tag = "href_link" then
								Web.Write_Begin_Attribute (Output, "href");
								Web.Write_In_Attribute (
									Output,
									Web.XHTML,
									Watched_Syndication.Info.Link.Constant_Reference);
								Web.Write_End_Attribute (Output);
							elsif Tag = "editable" then
								if User_Id = View_User_Id then
									declare
										procedure Handle (
											Output : not null access Ada.Streams.Root_Stream_Type'Class;
											Tag : in String;
											Contents : in Web.Producers.Template) is
										begin
											if Tag = "value_uri" then
												Web.Write_Begin_Attribute (Output, "value");
												Web.Write_In_Attribute (
													Output,
													Web.XHTML,
													Watched_Syndication.Info.URI.Constant_Reference);
												Web.Write_End_Attribute (Output);
											else
												Raise_Unknown_Tag (Tag);
											end if;
										end Handle;
									begin
										Web.Producers.Produce (Output, Contents, Handler => Handle'Access);
									end;
								end if;
							else
								Raise_Unknown_Tag (Tag);
							end if;
						end Handle;
					begin
						Web.Producers.Produce (Output, Contents, Handler => Handle'Access);
					end;
				end loop;
			end;
		elsif Tag = "href_rss" then
			Web.Write_Begin_Attribute (Output, "href");
			Web.Write_Query_In_Attribute (
				Output,
				Web.XHTML,
				Pages.Parameters_To_RSS_Page (View_User_Id));
			Web.Write_End_Attribute (Output);
		elsif Tag = "href_index" then
			Web.Write_Begin_Attribute (Output, "href");
			declare
				S : String
					renames Ada.Hierarchical_File_Names.Unchecked_Simple_Name (
						Web.Request_Path);
			begin
				if S'Length = 0 then
					Web.Write_In_Attribute (Output, Web.XHTML, "./");
				else
					Web.Write_In_Attribute (Output, Web.XHTML, S);
				end if;
			end;
			Web.Write_Query_In_Attribute (
				Output,
				Web.XHTML,
				Pages.Parameters_To_Index_Page);
			Web.Write_End_Attribute (Output);
		elsif Tag = "editable" then
			if User_Id = View_User_Id then
				Web.Producers.Produce (Output, Contents);
			end if;
		else
			Raise_Unknown_Tag (Tag);
		end if;
	end Handle;
begin
	Web.Producers.Produce (Output, Read (Template), Handler => Handle'Access);
end Birdeyes.Pages.View_Page;
