with Ada.Strings.Unbounded;
with Web.Producers;
with Birdeyes.Users.Management;
package Birdeyes.Pages is
	
	type Page is (Index, View, RSS);
	subtype Base_Page is Page range Index .. View;
	
	function Parameters_To_Index_Page
		return Web.Query_Strings;
	
	function Parameters_To_View_Page (
		View_User_Id : String)
		return Web.Query_Strings;
	
	function Parameters_To_RSS_Page (
		View_User_Id : String)
		return Web.Query_Strings;
	
	function Where (
		Query_Strings : Web.Query_Strings)
		return Page;
	
	function Get_View_User_Id (
		Query_Strings : Web.Query_Strings;
		User_Manager : Users.Management.Manager)
		return String;
	
	-- user
	
	function Get_User_Id (
		Cookie : Web.Cookie;
		User_Manager : Users.Management.Manager)
		return String;
	
	function Get_User_Password (
		Cookie : Web.Cookie)
		return String;
	
	procedure Set_User (
		Cookie : in out Web.Cookie;
		New_User_Id: in String;
		New_User_Password : in String);
	
	-- from form
	
	function Get_Command (
		Inputs : Web.Query_Strings)
		return String;
	
	function Get_New_User_Id (
		Inputs : Web.Query_Strings)
		return String;
	
	function Get_New_User_Password (
		Inputs : Web.Query_Strings)
		return String;
	
	function Get_New_User_Confirmation_Password (
		Inputs : Web.Query_Strings)
		return String;
	
	function Get_URI (
		Inputs : Web.Query_Strings)
		return String;
	
	function Get_Showing (
		Inputs : Web.Query_Strings)
		return Users.Showing_Mode;
	
	function Get_Content (
		Inputs : Web.Query_Strings)
		return String;
	
	-- message
	
	function Has_Message (Cookie : Web.Cookie) return Boolean;
	
	procedure Get_Message (
		Cookie : in out Web.Cookie;
		Message : out Ada.Strings.Unbounded.Unbounded_String);
	
	procedure Set_Message (
		Cookie : in out Web.Cookie;
		Message : in String);
	
private
	
	function Read (
		Template_Source : in String;
		Template_Cache : in String := "")
		return Web.Producers.Template;
	
	procedure Raise_Unknown_Tag (
		Tag : in String;
		File : in String := Ada.Debug.File;
		Line : in Integer := Ada.Debug.Line);
	pragma No_Return (Raise_Unknown_Tag);
	
end Birdeyes.Pages;
