with Ada.Calendar;
with Ada.Containers.Doubly_Linked_Lists;
with Ada.Streams;
with Ada.Strings.Unbounded;
with Birdeyes.Calendar;
package Birdeyes.Syndications is
	use type Ada.Calendar.Time;
	
	type Syndication_Version is (Unknown, RSS_1, RSS_2, Atom);
	
	function Image (Value : Syndication_Version) return String;
	
	type History is record
		Modified_Time : Ada.Calendar.Time;
	end record;
	
	Default_History : constant History := (Modified_Time => Calendar.Null_Time);
	
	package History_Lists is
		new Ada.Containers.Doubly_Linked_Lists (History);
	
	type Syndication_Info is record
		URI : aliased Ada.Strings.Unbounded.Unbounded_String;
		Version : Syndication_Version;
		Link : aliased Ada.Strings.Unbounded.Unbounded_String;
		Title : aliased Ada.Strings.Unbounded.Unbounded_String;
		Author : aliased Ada.Strings.Unbounded.Unbounded_String;
		Content_Length : Ada.Streams.Stream_Element_Count := 0;
		Last_Tried_Time : Ada.Calendar.Time := Calendar.Null_Time;
		Last_Avail_Time : Ada.Calendar.Time := Calendar.Null_Time;
		Last_Detected_Time : Ada.Calendar.Time := Calendar.Null_Time;
		History : aliased History_Lists.List;
	end record;
	
	Modified_Time_Limit : constant Duration := 365.0 * 24.0 * 60.0 * 60.0;
	
	procedure Add_History (
		Container : in out History_Lists.List;
		Modified_Time : in Ada.Calendar.Time);
	
	function Last_Modified_Time (Info : Syndication_Info)
		return Ada.Calendar.Time;
	
	type Article is record
		Link : aliased Ada.Strings.Unbounded.Unbounded_String;
		Title : aliased Ada.Strings.Unbounded.Unbounded_String;
		Author : aliased Ada.Strings.Unbounded.Unbounded_String;
		Description : aliased Ada.Strings.Unbounded.Unbounded_String;
		Content : aliased Ada.Strings.Unbounded.Unbounded_String;
		Source_URI : aliased Ada.Strings.Unbounded.Unbounded_String;
		Source_Title : aliased Ada.Strings.Unbounded.Unbounded_String;
		Time : Ada.Calendar.Time := Calendar.Null_Time;
	end record;
	
	package Article_Lists is
		new Ada.Containers.Doubly_Linked_Lists (Article);
	
	function Find_Latest (Container : Article_Lists.List; Time : Ada.Calendar.Time)
		return Article_Lists.Cursor;
	
	procedure Read_RSS (
		Stream : not null access Ada.Streams.Root_Stream_Type'Class;
		Info : in out Syndication_Info;
		Articles : out Article_Lists.List);
	
	type Syndication_Info_And_Articles is record
		Info : Syndication_Info;
		Articles : aliased Article_Lists.List;
	end record;
	
	package Syndication_Info_And_Articles_Lists is
		new Ada.Containers.Doubly_Linked_Lists (Syndication_Info_And_Articles);
	
	Not_RSS : exception;
	
end Birdeyes.Syndications;
