with Ada.IO_Exceptions;
package Birdeyes.Users.Management is
	
	type Manager (<>) is limited private;
	
	function Create (Path : Static_String_Access) return Manager;
	
	type User_Status is (Log_Off, Valid);
	
	procedure Iterate (
		Object : in Manager;
		Process : not null access procedure (Id : in String));
	
	function Exists (Object : Manager; Id : String) return Boolean;
	
	-- fix casing
	function Registered_Id (Object : Manager; Id : String) return String;
	
	procedure Add (
		Object : in out Manager;
		Id : in String;
		Password : in Password_Digest;
		Remote_Addr : in String;
		Remote_Host : in String;
		Time : in Ada.Calendar.Time;
		Info : out User_Info);
	
	procedure Query (
		Object : in out Manager;
		Id : in String;
		Password : in Password_Digest;
		Remote_Addr : in String;
		Remote_Host : in String;
		Time : in Ada.Calendar.Time;
		Info : out User_Info);
	
	-- query only watching list
	procedure Query (
		Object : in out Manager;
		Id : in String;
		Watching : out Watching_Syndications.List);
	
	procedure Update (
		Object : in out Manager;
		Id : in String;
		Info : in out User_Info);
	
	Name_Error : exception
		renames Ada.IO_Exceptions.Name_Error;
	Password_Mismatch : exception;
	
private
	
	type Manager is limited record
		Path : Static_String_Access;
	end record;
	
end Birdeyes.Users.Management;
