with Ada.Strings.Functions;
with XML.Streams;
with Web;
package body Birdeyes.Users.View is
	use type Ada.Calendar.Time;
	use type Syndications.Article_Lists.Cursor;
	
	procedure Append_Article (
		Container : in out Syndications.Article_Lists.List;
		Watched_Syndication : in Syndications.Syndication_Info_And_Articles;
		Article : in Syndications.Article;
		Showing : in Showing_Mode)
	is
		New_Item : Syndications.Article := Article;
	begin
		if New_Item.Author.Is_Null then
			New_Item.Author := Watched_Syndication.Info.Author;
		end if;
		if New_Item.Source_URI.Is_Null then
			New_Item.Source_URI := Watched_Syndication.Info.URI;
			New_Item.Source_Title := Watched_Syndication.Info.Title;
		end if;
		declare
			Last_Modified_Time : Ada.Calendar.Time :=
				Syndications.Last_Modified_Time (Watched_Syndication.Info);
		begin
			if Last_Modified_Time /= Calendar.Null_Time
				and then New_Item.Time > Last_Modified_Time
			then
				New_Item.Time := Last_Modified_Time;
			end if;
		end;
		case Showing is
			when Latest =>
				New_Item.Link := Watched_Syndication.Info.Link;
				New_Item.Title := Watched_Syndication.Info.Title;
			when Expanded =>
				if not New_Item.Author.Is_Null then
					Ada.Strings.Unbounded.Append (New_Item.Author, " - ");
				end if;
				Ada.Strings.Unbounded.Append (New_Item.Author, Watched_Syndication.Info.Title);
		end case;
		Syndications.Article_Lists.Append (Container, New_Item);
	end Append_Article;
	
	Advertisement : constant array (Positive range <>) of Static_String_Access :=
		(1 => new String'("http://rss.rssad.jp/rss/ad/"));
	
	function Is_Advertisement (URI : String) return Boolean is
	begin
		for I in Advertisement'Range loop
			if Advertisement (I)'Length <= URI'Length
				and then Advertisement (I).all =
				         URI (URI'First .. URI'First + Advertisement (I)'Length - 1)
			then
				return True;
			end if;
		end loop;
		return False;
	end Is_Advertisement;
	
	Limit_Of_One_Article : constant := 1024;
	
	function Strip_Tag (S : String)
		return Ada.Strings.Unbounded.Unbounded_String is
	begin
		return Result : Ada.Strings.Unbounded.Unbounded_String do
			declare
				I : Positive := S'First;
			begin
				while I <= S'Last loop
					declare
						Left : Natural := Ada.Strings.Functions.Index_Element (S (I .. S'Last), '<');
						Right : Natural;
					begin
						exit when Left = 0;
						Ada.Strings.Unbounded.Append (Result, S (I .. Left - 1));
						if Result.Length >= Limit_Of_One_Article then
							goto Limit;
						end if;
						Right := Left + 1;
						while Right <= S'Last and then S (Right) /= '>' loop
							Right := Right + 1;
						end loop;
						I := Right + 1;
					end;
				end loop;
				Ada.Strings.Unbounded.Append (Result, S (I .. S'Last));
			end;
		<<Limit>>
			if Result.Length > Limit_Of_One_Article then
				declare
					I : Positive := Limit_Of_One_Article + 1;
				begin
					while Result.Element (I) in
						Character'Val (2#10000000#) .. Character'Val (2#10111111#)
					loop
						I := I - 1;
					end loop;
					Ada.Strings.Unbounded.Delete (Result, I, Result.Length);
				end;
			end if;
		end return;
	end Strip_Tag;
	
	function "<" (Left, Right : Syndications.Syndication_Info_And_Articles)
		return Boolean is
	begin
		return Syndications.Last_Modified_Time (Left.Info) >
			Syndications.Last_Modified_Time (Right.Info);
			-- reverse
	end "<";
	
	package Syndication_Info_And_Articles_Sorting is
		new Syndications.Syndication_Info_And_Articles_Lists.Generic_Sorting;
	
	function "<" (Left, Right : Syndications.Article) return Boolean is
	begin
		return Left.Time > Right.Time; -- reverse
	end "<";
	
	package Article_Sorting is new Syndications.Article_Lists.Generic_Sorting;
	
	-- implementation
	
	procedure Sort (
		Watching : in Watching_Syndications.List;
		Watched_Syndications : in out
			Syndications.Syndication_Info_And_Articles_Lists.List;
		Time : in Ada.Calendar.Time) is
	begin
		for I in Watched_Syndications.Iterate loop
			declare
				Watched_Syndication : Syndications.Syndication_Info_And_Articles
					renames Watched_Syndications.Reference (I);
				Watching_Position : Users.Watching_Syndications.Cursor :=
					Users.Find (Watching, Watched_Syndication.Info.URI.Constant_Reference);
			begin
				pragma Assert (Users.Watching_Syndications.Has_Element (Watching_Position));
				case Watching.Constant_Reference (Watching_Position).Showing is
					when Latest =>
						declare
							J : Syndications.Article_Lists.Cursor :=
								Syndications.Find_Latest (Watched_Syndication.Articles, Time);
						begin
							if Syndications.Article_Lists.Has_Element (J)
								and then J /= Watched_Syndication.Articles.First
							then
								Syndications.Article_Lists.Swap_Links (
									Watched_Syndication.Articles,
									J,
									Watched_Syndication.Articles.First);
							end if;
						end;
					when Expanded =>
						declare
							J : Syndications.Article_Lists.Cursor :=
								Watched_Syndication.Articles.First;
						begin
							while Syndications.Article_Lists.Has_Element (J) loop
								declare
									Next : constant Syndications.Article_Lists.Cursor :=
										Syndications.Article_Lists.Next (J);
									Article : Syndications.Article
										renames Watched_Syndication.Articles.Constant_Reference (J);
								begin
									if Is_Advertisement (Article.Link.Constant_Reference) then
										Syndications.Article_Lists.Delete (Watched_Syndication.Articles, J);
									end if;
									J := Next;
								end;
							end loop;
						end;
						Article_Sorting.Sort (Watched_Syndication.Articles);
				end case;
			end;
		end loop;
		Syndication_Info_And_Articles_Sorting.Sort (Watched_Syndications);
	end Sort;
	
	procedure Fold (
		Watching : in Watching_Syndications.List;
		Watched_Syndications : in
			Syndications.Syndication_Info_And_Articles_Lists.List;
		Time : in Ada.Calendar.Time;
		Result : out Syndications.Article_Lists.List) is
	begin
		for I in Watched_Syndications.Iterate loop
			declare
				Watched_Syndication : Syndications.Syndication_Info_And_Articles
					renames Watched_Syndications.Constant_Reference (I);
				Watching_Position : Users.Watching_Syndications.Cursor :=
					Users.Find (Watching, Watched_Syndication.Info.URI.Constant_Reference);
			begin
				pragma Assert (Users.Watching_Syndications.Has_Element (Watching_Position));
				case Watching.Constant_Reference (Watching_Position).Showing is
					when Latest =>
						declare
							J : Syndications.Article_Lists.Cursor :=
								Syndications.Find_Latest (Watched_Syndication.Articles, Time);
						begin
							if Syndications.Article_Lists.Has_Element (J) then
								Append_Article (
									Result,
									Watched_Syndication,
									Watched_Syndication.Articles.Constant_Reference (J),
									Latest);
							end if;
						end;
					when Expanded =>
						for J in Watched_Syndication.Articles.Iterate loop
							declare
								Article : Syndications.Article
									renames Watched_Syndication.Articles.Constant_Reference (J);
							begin
								if not Is_Advertisement (Article.Link.Constant_Reference) then
									Append_Article (
										Result,
										Watched_Syndication,
										Article,
										Expanded);
								end if;
							end;
						end loop;
				end case;
			end;
		end loop;
		Article_Sorting.Sort (Result);
	end Fold;
	
	RSS_Tag : aliased constant String := "rss";
	Version_Attribute : aliased constant String := "version";
	Version_2_Value : aliased constant String := "2.0";
	
	Channel_Tag : aliased constant String := "channel";
	
	Title_Tag : aliased constant String := "title";
	
	Link_Tag : aliased constant String := "link";
	
	PubDate_Tag : aliased constant String := "pubDate";
	
	Generator_Tag : aliased constant String := "generator";
	
	Docs_Tag : aliased constant String := "docs";
	Docs_Value : aliased constant String :=
		"http://blogs.law.harvard.edu/tech/rss";
	
	Item_Tag : aliased constant String := "item";
	
	Author_Tag : aliased constant String := "author";
	
	Description_Tag : aliased constant String := "description";
	
	Source_Tag : aliased constant String := "source";
	URL_Attribute : aliased constant String := "url";
	
	procedure Write_RSS (
		Stream : not null access Ada.Streams.Root_Stream_Type'Class;
		Generator : in String := Generator_Mark;
		Link : in String := "";
		Title : in String := "";
		Articles : in Syndications.Article_Lists.List)
	is
		Writer : XML.Writer :=
			XML.Streams.Create (Stream, Encoding => XML.Find ("UTF-8"));
	begin
		XML.Write (Writer, (Event_Type => XML.Element_Start, Name => RSS_Tag'Access));
		XML.Write (
			Writer,
			(Event_Type => XML.Attribute,
				Name => Version_Attribute'Access,
				Value => Version_2_Value'Access));
		XML.Write (
			Writer,
			(Event_Type => XML.Element_Start, Name => Channel_Tag'Access));
		if Link'Length /= 0 then
			XML.Write (Writer, (Event_Type => XML.Element_Start, Name => Link_Tag'Access));
			XML.Write (
				Writer,
				(Event_Type => XML.Text, Content => Link'Unrestricted_Access));
			XML.Write (Writer, (Event_Type => XML.Element_End));
		end if;
		if Title'Length /= 0 then
			XML.Write (
				Writer,
				(Event_Type => XML.Element_Start, Name => Title_Tag'Access));
			XML.Write (
				Writer,
				(Event_Type => XML.Text, Content => Title'Unrestricted_Access));
			XML.Write (Writer, (Event_Type => XML.Element_End));
		end if;
		if not Articles.Is_Empty
			and then Articles.Constant_Reference (Articles.First).Time /=
			         Calendar.Null_Time
		then
			declare
				S : aliased constant String :=
					Web.Image (Articles.Constant_Reference (Articles.First).Time);
			begin
				XML.Write (
					Writer,
					(Event_Type => XML.Element_Start, Name => PubDate_Tag'Access));
				XML.Write (Writer, (Event_Type => XML.Text, Content => S'Unchecked_Access));
				XML.Write (Writer, (Event_Type => XML.Element_End));
			end;
		end if;
		if Generator'Length /= 0 then
			XML.Write (
				Writer,
				(Event_Type => XML.Element_Start, Name => Generator_Tag'Access));
			XML.Write (
				Writer,
				(Event_Type => XML.Text, Content => Generator'Unrestricted_Access));
			XML.Write (Writer, (Event_Type => XML.Element_End));
		end if;
		XML.Write (Writer, (Event_Type => XML.Element_Start, Name => Docs_Tag'Access));
		XML.Write (Writer, (Event_Type => XML.Text, Content => Docs_Value'Access));
		XML.Write (Writer, (Event_Type => XML.Element_End));
		for I in Articles.Iterate loop
			declare
				Article : Syndications.Article
					renames Articles.Constant_Reference (I);
			begin
				XML.Write (Writer, (Event_Type => XML.Element_Start, Name => Item_Tag'Access));
				if not Article.Link.Is_Null then
					XML.Write (Writer, (Event_Type => XML.Element_Start, Name => Link_Tag'Access));
					XML.Write (
						Writer,
						(Event_Type => XML.Text, Content => Article.Link.Constant_Reference.Element));
					XML.Write (Writer, (Event_Type => XML.Element_End));
				end if;
				if not Article.Title.Is_Null then
					XML.Write (
						Writer,
						(Event_Type => XML.Element_Start, Name => Title_Tag'Access));
					XML.Write (
						Writer,
						(Event_Type => XML.Text, Content => Article.Title.Constant_Reference.Element));
					XML.Write (Writer, (Event_Type => XML.Element_End));
				end if;
				if not Article.Author.Is_Null then
					XML.Write (
						Writer,
						(Event_Type => XML.Element_Start, Name => Author_Tag'Access));
					XML.Write (
						Writer,
						(Event_Type => XML.Text,
							Content => Article.Author.Constant_Reference.Element));
					XML.Write (Writer, (Event_Type => XML.Element_End));
				end if;
				if not Article.Description.Is_Null then
					declare
						S : aliased constant Ada.Strings.Unbounded.Unbounded_String :=
							Strip_Tag (Article.Description.Constant_Reference);
					begin
						XML.Write (
							Writer,
							(Event_Type => XML.Element_Start, Name => Description_Tag'Access));
						XML.Write (
							Writer,
							(Event_Type => XML.CDATA, Content => S.Constant_Reference.Element));
						XML.Write (Writer, (Event_Type => XML.Element_End));
					end;
				end if;
				if not Article.Source_URI.Is_Null then
					XML.Write (
						Writer,
						(Event_Type => XML.Element_Start, Name => Source_Tag'Access));
					XML.Write (Writer, (
						Event_Type => XML.Attribute,
						Name => URL_Attribute'Access,
						Value => Article.Source_URI.Constant_Reference.Element));
					XML.Write (Writer, (
						Event_Type => XML.Text,
						Content => Article.Source_Title.Constant_Reference.Element));
					XML.Write (Writer, (Event_Type => XML.Element_End));
				end if;
				if Article.Time /= Calendar.Null_Time then
					declare
						S : aliased constant String := Web.Image (Article.Time);
					begin
						XML.Write (
							Writer,
							(Event_Type => XML.Element_Start, Name => PubDate_Tag'Access));
						XML.Write (Writer, (Event_Type => XML.Text, Content => S'Unchecked_Access));
						XML.Write (Writer, (Event_Type => XML.Element_End));
					end;
				end if;
				XML.Write (Writer, (Event_Type => XML.Element_End));
			end;
		end loop;
		XML.Write (Writer, (Event_Type => XML.Element_End));
		XML.Write (Writer, (Event_Type => XML.Element_End));
	end Write_RSS;
	
end Birdeyes.Users.View;
