with Crypto.SHA1;
package body Birdeyes.Users is
	use type Ada.Strings.Unbounded.Unbounded_String;
	
	pragma Compile_Time_Error (
		Password_Digest'Length /= Crypto.SHA1.Fingerprint'Length,
		"length of SHA1 /= 20");
	
	pragma Compile_Time_Error (
		Password_Digest_Image'Length /= Crypto.SHA1.Message_Digest'Length,
		"image length of SHA1 /= 40");
	
	-- implementation
	
	function Valid_Id_String (Id : String) return Boolean is
	begin
		if Id'Length = 0 then
			return False;
		end if;
		for I in Id'Range loop
			case Id (I) is
				when 'A'..'Z' | 'a'..'z' | '0'..'9' | '_' | '-' =>
					null;
				when others =>
					return False;
			end case;
		end loop;
		return True;
	end Valid_Id_String;
	
	function Digest (Password : String) return Password_Digest is
	begin
		return Result : Password_Digest do
			declare
				SHA1 : Crypto.SHA1.Context := Crypto.SHA1.Initial;
			begin
				Crypto.SHA1.Update (SHA1, Password);
				Crypto.SHA1.Final (SHA1, Crypto.SHA1.Fingerprint (Result));
			end;
		end return;
	end Digest;
	
	function Image (Digest : Password_Digest) return Password_Digest_Image is
	begin
		return Crypto.SHA1.Image (Crypto.SHA1.Fingerprint (Digest));
	end Image;
	
	function Value (Image : Password_Digest_Image) return Password_Digest is
	begin
		return Password_Digest (Crypto.SHA1.Value (Image));
	end Value;
	
	function Find (Container : Watching_Syndications.List; URI : String)
		return Watching_Syndications.Cursor is
	begin
		for I in Container.Iterate loop
			if Container.Constant_Reference (I).URI = URI then
				return I;
			end if;
		end loop;
		return Watching_Syndications.No_Element;
	end Find;
	
	function Contains (Container : Watching_Syndications.List; URI : String)
		return Boolean is
	begin
		return Watching_Syndications.Has_Element (Find (Container, URI));
	end Contains;
	
	procedure Delete (
		Container : in out Watching_Syndications.List;
		URI : in String)
	is
		I : Watching_Syndications.Cursor := Find (Container, URI);
	begin
		if not Watching_Syndications.Has_Element (I) then
			raise Constraint_Error;
		else
			Watching_Syndications.Delete (Container, I);
		end if;
	end Delete;
	
	function To_URI_List (Container : Watching_Syndications.List)
		return URI.Lists.List is
	begin
		return Result : URI.Lists.List do
			for I in Container.Iterate loop
				URI.Lists.Append (Result, Container.Constant_Reference (I).URI);
			end loop;
		end return;
	end To_URI_List;
	
end Birdeyes.Users;
