with Ada.Calendar;
with Ada.Containers.Doubly_Linked_Lists;
with Ada.Strings.Unbounded;
with Birdeyes.Calendar;
with Birdeyes.URI;
private with Ada.Streams;
package Birdeyes.Users is
	
	-- id
	
	function Valid_Id_String (Id : String) return Boolean;
	
	-- password
	
	type Password_Digest is private;
	
	Null_Password_Digest : constant Password_Digest;
	
	function Digest (Password : String) return Password_Digest;
	
	subtype Password_Digest_Image is String (1 .. 40);
	
	function Image (Digest : Password_Digest) return Password_Digest_Image;
	function Value (Image : Password_Digest_Image) return Password_Digest;
	
	-- watching
	
	type Showing_Mode is (Latest, Expanded);
	
	type Watching_Syndication is record
		URI : Ada.Strings.Unbounded.Unbounded_String;
		Showing : Showing_Mode;
	end record;
	
	Default_Watching_Syndication : constant Watching_Syndication :=
		(URI => Ada.Strings.Unbounded.Null_Unbounded_String, Showing => Latest);
	
	package Watching_Syndications is
		new Ada.Containers.Doubly_Linked_Lists (Watching_Syndication);
	
	function Find (Container : Watching_Syndications.List; URI : String)
		return Watching_Syndications.Cursor;
	
	function Contains (Container : Watching_Syndications.List; URI : String)
		return Boolean;
	
	procedure Delete (
		Container : in out Watching_Syndications.List;
		URI : in String);
	
	function To_URI_List (Container : Watching_Syndications.List)
		return URI.Lists.List;
	
	-- IP address
	Local_Host_Addr : constant String := "127.0.0.1";
	Local_Host_Name : constant String := "localhost";
	
	-- user info
	
	type User_Info is record
		-- password (SHA-1)
		Password : Password_Digest := Null_Password_Digest;
		-- about creation
		Creation_Remote_Addr : Ada.Strings.Unbounded.Unbounded_String;
		Creation_Remote_Host : Ada.Strings.Unbounded.Unbounded_String;
		Creation_Time : Ada.Calendar.Time := Calendar.Null_Time;
		-- about last access
		Last_Remote_Addr : Ada.Strings.Unbounded.Unbounded_String;
		Last_Remote_Host : Ada.Strings.Unbounded.Unbounded_String;
		Last_Time : Ada.Calendar.Time := Calendar.Null_Time;
		-- watching
		Watching : Watching_Syndications.List;
	end record;
	
private
	
	type Password_Digest is new Ada.Streams.Stream_Element_Array (0 .. 19);
	
	Null_Password_Digest : constant Password_Digest := (others => 0);
	
end Birdeyes.Users;
