with Ada.Directories.Temporary;
with Ada.Exceptions;
with Ada.Streams.Stream_IO;
with Serialization.YAML;
with YAML.Streams;
package body Birdeyes.YAML_IO is
	
	procedure Load (Name : in String; Item : in out T) is
		File : Ada.Streams.Stream_IO.File_Type :=
			Ada.Streams.Stream_IO.Open (Ada.Streams.Stream_IO.In_File, Name => Name);
	begin
		declare
			Parser : aliased YAML.Parser :=
				YAML.Streams.Create (Ada.Streams.Stream_IO.Stream (File));
		begin
			YAML.Parse_Stream_Start (Parser);
			IO (Serialization.YAML.Reading (Parser'Access, Tag).Serializer, Item);
			YAML.Parse_Stream_End (Parser);
		end;
		Ada.Streams.Stream_IO.Close (File);
	exception
		when E : others =>
			Ada.Debug.Put ("on reading " & Name);
			Ada.Debug.Put (Ada.Exceptions.Exception_Information (E));
			declare
				Message : constant String :=
					Name & ": " & Ada.Exceptions.Exception_Message (E);
			begin
				Ada.Exceptions.Raise_Exception (
					Ada.Exceptions.Exception_Identity (E),
					Message);
			end;
	end Load;
	
	procedure Save (Name : in String; Item : in out T) is
		Temporary_Name : constant String :=
			Ada.Directories.Temporary.Create_Temporary_File;
		File : Ada.Streams.Stream_IO.File_Type :=
			Ada.Streams.Stream_IO.Create (Name => Temporary_Name);
	begin
		declare
			Emitter : aliased YAML.Emitter :=
				YAML.Streams.Create (Ada.Streams.Stream_IO.Stream (File));
		begin
			YAML.Set_Unicode (Emitter, True);
			YAML.Emit (Emitter, (Event_Type => YAML.Stream_Start, Encoding => YAML.UTF_8));
			IO (Serialization.YAML.Writing (Emitter'Access, Tag).Serializer, Item);
			YAML.Emit (Emitter, (Event_Type => YAML.Stream_End));
			YAML.Flush (Emitter);
		end;
		Ada.Streams.Stream_IO.Close (File);
		Ada.Directories.Replace_File (
			Source_Name => Temporary_Name,
			Target_Name => Name);
	exception
		when E : others =>
			Ada.Debug.Put ("on writing " & Name & " (" & Temporary_Name & ")");
			Ada.Debug.Put (Ada.Exceptions.Exception_Information (E));
			declare
				Message : constant String :=
					Name & ": " & Ada.Exceptions.Exception_Message (E);
			begin
				Ada.Exceptions.Raise_Exception (
					Ada.Exceptions.Exception_Identity (E),
					Message);
			end;
	end Save;
	
end Birdeyes.YAML_IO;
